#include "stdafx.h"
#include "PaintProcs.h"
#include "Main.h"
#include "Interface.h"
#include <vector>

// int g_mouse_x;
// int g_mouse_y;
const int c_paint_object_period_ms=20;

static const int c_player_width=10;
static const int c_player_hight=36;
static const int c_player_move_step=2;
static const int c_shoot_relaxation=10;

static const int c_bullet_cout=8;
static const int c_bullet_width=8;
static const int c_bullet_hight=8;
static const int c_bullet_move_step=6;

static const int c_player_1_x=580;
static const int c_player_1_y=170;
static const int c_player_2_x=10;
static const int c_player_2_y=170;

enum MoveType{mt_move_stop, mt_move_up, mt_move_down};

struct CommandState
{
	MoveType move_type;
	bool shoot;
};

struct Player
{
	Player()
	{
		command.move_type=mt_move_stop;
		command.shoot=false;

		shoot_relaxation=0;
		id=0;
		explode=false;
	}

	POINT pos;
	int shoot_relaxation;
	int id;
	bool explode;

	CommandState command;
	IPlayer* player;//  NULL- 
};

struct Bullet
{
	Bullet(POINT pos_, bool direction_left_)
	{
		pos=pos_;
		direction_left=direction_left_;
		explode=false;
	}

	POINT pos;
	bool direction_left;
	bool explode;
};

std::vector<Bullet> g_bullets1;
std::vector<Bullet> g_bullets2;

Player g_player1;
Player g_player2;
static int g_called_player;

Player* GetPlayer(size_t player_index)
{
	if(player_index==0)
		return &g_player1;
	if(player_index==1)
		return &g_player2;

	return NULL;
}

std::vector<Bullet>* GetBullets(int player_index)
{
	if(player_index==0)
		return &g_bullets1;
	if(player_index==1)
		return &g_bullets2;

	return NULL;
}

class Host: public IHost
{
	virtual void GetSelfIndex(size_t* index)
	{
		*index=g_called_player;
	}

	virtual void GetPlayerPos(size_t player_index, POINT* pos)
	{
		if(player_index>1)
			return;
		*pos=GetPlayer(player_index)->pos;
	}

	virtual void GetBulletCount(size_t player_index, size_t* count)
	{
		if(player_index>1)
			return;
		*count=GetBullets(player_index)->size();
	}

	virtual void GetBulletPos(size_t player_index, size_t bullet_index, POINT* pos)
	{
		std::vector<Bullet>* bullets;

		if(player_index>1)
			return;

		bullets=GetBullets(player_index);
		if(bullet_index>=bullets->size())
			return;

		*pos=(*bullets)[bullet_index].pos;
	}

	virtual void Action(ActionType action)
	{
		Player* player=GetPlayer(g_called_player); 

		switch(action)
		{
		case at_move_up: player->command.move_type=mt_move_up; break;
		case at_move_down: player->command.move_type=mt_move_down; break;
		case at_move_stop: player->command.move_type=mt_move_stop; break;
		case at_shoot: player->command.shoot=true; break;
		case at_shoot_stop: player->command.shoot=false; break;
		}
	}

};

Host	g_host;

extern "C" void IHostRegisterPlayer(IPlayer* player)
{
	if(g_player1.player==NULL)
		g_player1.player=player;
	else if(g_player2.player==NULL)
		g_player2.player=player;
}


void PlayerPosValidate(Player* player, int width, int hight)
{
	if(player->pos.y + c_player_hight/2 >= hight)
		player->pos.y=hight-c_player_hight/2;
	else if(player->pos.y - c_player_hight/2 <= 0)
		player->pos.y=c_player_hight/2;
}

RECT PlayerRect(const Player& player)
{
	RECT rect;
	rect.left = player.pos.x - c_player_width/2;
	rect.right = player.pos.x + c_player_width/2;
	rect.top = player.pos.y - c_player_hight/2;
	rect.bottom = player.pos.y + c_player_hight/2;

	return rect;
}

RECT BulletRect(const Bullet& bullet)
{
	RECT rect;

	rect.left = bullet.pos.x - c_bullet_width/2;
	rect.right = bullet.pos.x + c_bullet_width/2;
	rect.top = bullet.pos.y - c_bullet_hight/2;
	rect.bottom = bullet.pos.y + c_bullet_hight/2;

	return rect;
}

void PlayerBulletCollide(Player* player, std::vector<Bullet>* bullets)
{
	RECT player_rect=PlayerRect(*player);

	for(size_t i=0; i<bullets->size(); i++)
	{
		Bullet& bullet=(*bullets)[i];
		RECT bullet_rect=BulletRect(bullet);
		RECT tmp;

		if(TRUE == IntersectRect(&tmp, &player_rect, &bullet_rect) )
		{
			bullet.explode=true;
			player->explode=true;
		}
	}
}

void BulletExplode(std::vector<Bullet>* bullets)
{
	for(std::vector<Bullet>::iterator it=bullets->begin(); it!=bullets->end(); )
	{
		if(it->explode)
			it=bullets->erase(it);
		else
			it++;
	}
}

void BorderBulletCollide(int width, int hight, std::vector<Bullet>* bullets)
{
	RECT border_rect={0,0,width,hight};

	for(size_t i=0; i<bullets->size(); i++)
	{
		Bullet& bullet=(*bullets)[i];
		RECT bullet_rect=BulletRect(bullet);
		RECT in_view_rect;

		IntersectRect(&in_view_rect, &border_rect, &bullet_rect);
		if(FALSE==EqualRect(&in_view_rect,&bullet_rect))
		{
			bullet.explode=true;
		}
	}
}

void BulletsPaint(HDC dc, int width, int hight, const std::vector<Bullet>& bullets)
{
	for(size_t i=0; i<bullets.size(); i++)
	{
		RECT rect=BulletRect(bullets[i]);
		Ellipse(dc,rect.left, rect.top, rect.right, rect.bottom);
	}
}

void BulletsMove(std::vector<Bullet>* bullets)
{
	for(size_t i=0; i<bullets->size(); i++)
	{
		Bullet& bullet=(*bullets)[i];

		if(bullet.direction_left)
			bullet.pos.x-=c_bullet_move_step;
		else
			bullet.pos.x+=c_bullet_move_step;
	}
}


void PlayerMove(Player* player)
{
	switch(player->command.move_type)
	{
	case mt_move_up: player->pos.y-=c_player_move_step; break;
	case mt_move_down: player->pos.y+=c_player_move_step; break;
	case mt_move_stop: player->pos.y+=0; break;
	}
}

void PlayerStepRelaxation(Player* player)
{
	if(player->shoot_relaxation>0)
		player->shoot_relaxation--;
}

void PlayerShoot(Player* player, std::vector<Bullet>* bullets)
{
	if(player->explode)
		return;
	if(!player->command.shoot)
		return;
	if(player->shoot_relaxation!=0)
		return;
	if(bullets->size()>=c_bullet_cout)
		return;

	{
		bullets->push_back(Bullet(player->pos, (player->id==1) ) ) ;
		player->shoot_relaxation=c_shoot_relaxation;
	}
}

void PlayerPaint(HDC dc, int width, int hight, const Player& player)
{
	if(player.explode)
		return;

	RECT rect=PlayerRect(player);
	Rectangle(dc,rect.left, rect.top, rect.right, rect.bottom);
}

//-------------------------------------------------------

void PlayersInit()
{
	g_player1.id=1;
	g_player1.pos.x=c_player_1_x;
	g_player1.pos.y=c_player_1_y;

	g_player2.id=2;
	g_player2.pos.x=c_player_2_x;
	g_player2.pos.y=c_player_2_y;
}

class InitCaller
{
public:
	InitCaller()
	{
		PlayersInit();
	}
};

static InitCaller s_init;

void PaintObjects(HDC dc,  int width, int hight)
{
	SetTextColor(dc, RGB(255,255,255));
	SetBkMode(dc, TRANSPARENT);

	PaintText(dc, width/4,0, _T("Left Player: Q A C   Right Player: ? Up Down"));
// 	{
// 		char str[256];
// 		sprintf(str,"X %i",(int)(g_mouse_x));
// 		PaintText(dc, 0,0,str);
// 		sprintf(str,"Y %i",(int)(g_mouse_y));
// 		PaintText(dc, 60,0,str);
// 	}

// 	SetFillColor(0,255,0);
// 	SetFillColorEmpty();
// 
// 	SetContourLineColor(255,0,0);
// 	Ellipse(dc, 80, 90,  290, 225);

	PlayerMove(&g_player1);
	PlayerMove(&g_player2);
	PlayerShoot(&g_player1, &g_bullets1);
	PlayerShoot(&g_player2, &g_bullets2);
	BulletsMove(&g_bullets1);
	BulletsMove(&g_bullets2);

	PlayerStepRelaxation(&g_player1);
	PlayerStepRelaxation(&g_player2);
	PlayerPosValidate(&g_player1, width, hight);
	PlayerPosValidate(&g_player2, width, hight);

	PlayerBulletCollide(&g_player1, &g_bullets2);
	PlayerBulletCollide(&g_player2, &g_bullets1);
	BorderBulletCollide(width, hight, &g_bullets1);
	BorderBulletCollide(width, hight, &g_bullets2);

	BulletExplode(&g_bullets1);
	BulletExplode(&g_bullets2);

	PlayerPaint(dc, width, hight, g_player1);
	PlayerPaint(dc, width, hight, g_player2);

	BulletsPaint(dc, width, hight, g_bullets1);
	BulletsPaint(dc, width, hight, g_bullets2);

	if(g_player1.player!=NULL)
	{
		g_called_player=0;
		g_player1.player->Step(&g_host);
	}

	if(g_player2.player!=NULL)
	{
		g_called_player=1;
		g_player2.player->Step(&g_host);
	}
}

void OnKeyEvent(bool key_down, bool auto_repeat, int scan_code, int virtual_code)
{
	switch(virtual_code)
	{
	case VK_UP: g_player1.command.move_type=key_down?mt_move_up:mt_move_stop; break;
	case VK_DOWN: g_player1.command.move_type=key_down?mt_move_down:mt_move_stop; break;
	case VK_OEM_2: g_player1.command.shoot=key_down; break;

	case 'Q': g_player2.command.move_type=key_down?mt_move_up:mt_move_stop; break;
	case 'A': g_player2.command.move_type=key_down?mt_move_down:mt_move_stop; break;
	case 'C': g_player2.command.shoot=key_down; break;
	default:
	    break;
	}
}

void OnMouseEvent(MouseEventType event_type, int x, int y, DWORD key_indicators)
{
// 	g_mouse_x=x;
// 	g_mouse_y=y;
// 	PaintUpdateNow();
// 
// 	if(event_type==me_lbuttonup)
// 	{
// 		char str[256];
// 		sprintf(str,"%i, %i\r\n",(int)(g_mouse_x),(int)(g_mouse_y));
// 		OutputDebugString(str);
// 	}
// 	if(event_type==me_rbuttonup)
// 		OutputDebugString("-X----Y-\r\n");
}

